@@ -2,7 +2,7 @@
  *	avarice - The "avarice" program.
  *	Copyright (C) 2001 Scott Finneran
  *	Copyright (C) 2002, 2003, 2004 Intel Corporation
- *	Copyright (C) 2005,2006 Joerg Wunsch
+ *	Copyright (C) 2005, 2006, 2007 Joerg Wunsch
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License Version 2
@@ -19,7 +19,7 @@
  *
  * This file contains functions for interfacing with the GDB remote protocol.
  *
- * $Id: jtag.h,v 1.25 2006/10/28 20:17:31 joerg_wunsch Exp $
+ * $Id: jtag.h,v 1.27 2007/08/29 11:04:42 joerg_wunsch Exp $
  */
 
 #ifndef JTAG_H
@@ -435,6 +435,8 @@ enum
     EVT_TARGET_POWER_ON			= 0xE4,
     EVT_TARGET_SLEEP			= 0xE8,
     EVT_TARGET_WAKEUP			= 0xE9,
+    // trailer
+    EVT_MAX                             = 0xFF,
 
     // memory types for CMND_{READ,WRITE}_MEMORY
     MTYPE_IO_SHADOW	= 0x30,	// cached IO registers?
@@ -603,6 +605,9 @@ class jtag
   // For the mkII device, is the box attached via USB?
   bool is_usb;
 
+  // A control pipe to talk to the USB daemon.
+  int ctrlPipe;
+
   // The type of our emulator: JTAG ICE, or AVR Dragon.
   emulator emu_type;
 
@@ -707,6 +712,9 @@ class jtag
 
   virtual bool stopAt(unsigned int address) = 0;
 
+  /** Parse a list of event names to *not* cause a break. */
+  virtual void parseEvents(const char *) = 0;
+
   // Writing to program memory
   // -------------------------
 
